package com.hl.account;

import android.content.Context;
import android.text.TextUtils;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * Created by Administrator on 2017/8/15.
 */
public class AppUtil {
    
    /**
     * 判断email格式是否正确
     *
     * @param email
     * @return true：是邮箱
     */
    public static boolean isEmail(String email) {
        if (TextUtils.isEmpty(email)) {
            return false;
        }
        
        String str = "^([a-zA-Z0-9_\\-\\.]+)@((\\[[0-9]{1,3}\\.[0-9]{1,3}\\.[0-9]{1,3}\\.)|(" +
                "" + "([a-zA-Z0-9\\-]+\\.)+))([a-zA-Z]{2,4}|[0-9]{1,3})(\\]?)$";
        Pattern p = Pattern.compile(str);
        Matcher m = p.matcher(email);
        return m.matches();
    }
    
    /**
     * 判断密码是否合法
     */
    public static String isPwd(Context context, String pwd) {
        if (TextUtils.isEmpty(pwd)) {
            return context.getResources().getString(RTools.getString(context,
                    "lksea_please_input_password"));
            
        }
        
        if (pwd.contains(" ") || isChinese(pwd)) {
            return context.getString(RTools.getString(context, "lksea_password_not_comply_rules"));
        }
        
        int length = pwd.length();
        for (int i = 0; i < length; i++) {
            char charAt = pwd.charAt(i);
            boolean chinese = isChinese(charAt);
            if (chinese) {
                return context.getString(RTools.getString(context, "lksea_no_space"));
            }
        }
        
        if (length < 6 || length > 31) {
            return context.getString(RTools.getString(context, "lksea_length_error"));
            
        }
        
        String reg = "^[a-zA-Z0-9]+[.a-zA-Z0-9@_-]*$";
        Pattern pattern = Pattern.compile(reg);
        Matcher matcher = pattern.matcher(pwd);
        if (!matcher.matches()) {
            return context.getString(RTools.getString(context, "lksea_format_error"));
        }
        return null;
    }
    
    /**
     * 判断字符串是否是中文
     *
     * @param str 字符串
     */
    public static boolean isChinese(String str) {
        
        char[] ch = str.toCharArray();
        for (int i = 0; i < ch.length; i++) {
            char c = ch[i];
            if (isChinese(c)) {
                return true;
            }
        }
        return false;
    }
    
    
    // 根据Unicode编码完美的判断中文汉字和符号
    private static boolean isChinese(char c) {
        Character.UnicodeBlock ub = Character.UnicodeBlock.of(c);
        if (ub == Character.UnicodeBlock.CJK_UNIFIED_IDEOGRAPHS || ub == Character.UnicodeBlock
                .CJK_COMPATIBILITY_IDEOGRAPHS || ub == Character.UnicodeBlock
                .CJK_UNIFIED_IDEOGRAPHS_EXTENSION_A || ub == Character.UnicodeBlock
                .CJK_UNIFIED_IDEOGRAPHS_EXTENSION_B || ub == Character.UnicodeBlock
                .CJK_SYMBOLS_AND_PUNCTUATION || ub == Character.UnicodeBlock
                .HALFWIDTH_AND_FULLWIDTH_FORMS || ub == Character.UnicodeBlock
                .GENERAL_PUNCTUATION) {
            return true;
        }
        return false;
    }
    
    
    /**
     * 判断手机格式是否正确
     *
     * @param mobiles
     * @return true:是手机号
     */
    public static boolean isMobileNumber(String mobiles) {
        Pattern p = Pattern.compile("^((13[0-9])|(15[^4,\\D])|(18[0,5-9]))\\d{8}$");
        Matcher m = p.matcher(mobiles);
        return m.matches();
    }
    
    
    /**
     * 判断验证码长度是否为6位，并且全为数字
     *
     * @param verificationCode
     * @return 是否是验证码
     */
    public static boolean isVerification(String verificationCode) {
        if (verificationCode.length() != 6) {
            return false;
        }
        Pattern pattern = Pattern.compile("[0-9]*");
        Matcher matcher = pattern.matcher(verificationCode);
        return matcher.matches();
    }
    
    
    /**
     * 账号部分用*代替，隐藏内容
     *
     * @param passport
     * @return 186****1234
     */
    public static String showAccount(String passport) {
        
        int flag = passport.indexOf("@");
        String replaceSymbol = "*";
        StringBuilder stringBuilder = new StringBuilder();
        for (int i = 0; i < passport.length(); i++) {
            char number = passport.charAt(i);
            if (i >= 2 && i < flag - 2) {
                stringBuilder.append(replaceSymbol);
            } else {
                stringBuilder.append(number);
            }
        }
        return stringBuilder.toString();
    }
    
    public static int px2dip(Context context, float pxValue) {
        final float scale = context.getResources().getDisplayMetrics().density;
        return (int) (pxValue / scale + 0.5f);
    }
    
    /**
     * 根据手机的分辨率从 dp 的单位 转成为 px(像素)
     */
    public static int dip2px(Context context, float dpValue) {
        final float scale = context.getResources().getDisplayMetrics().density;
        return (int) (dpValue * scale + 0.5f);
    }
}
